/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   An access control list (ACL) that restricts access to an object on
   the local system.  The ACL contains a set of ACL entries that specify
   which principals may access the object and how.
   
   @see AclEntry
   
   @author Sameer Ajmani
 **/
public class Acl extends Obj
{
  private final AclEntry[] entries;

  public Acl(AclEntry[] e)
  {
    assert(e != null): "null entries";
    entries = e;
  }

  public AclEntry[] getEntries()
  {
    return entries;
  }

  public boolean equals(Object o)
  {
    return (o instanceof Acl)
      && Util.equals(entries, ((Acl)o).entries);
  }

  public int hashCode()
  {
    return Util.hashCode(entries);
  }

  public SexpList toSexp()
  {
    Sexp[] ss = new Sexp[entries.length];
    for (int i = 0; i < entries.length; i++) {
      ss[i] = entries[i].toSexp();
    }
    return SexpUtil.toSexp("acl", ss);
  }
  
  static Acl parseAcl(SexpList l)
    throws SexpParseException
  {
    Iterator abody = SexpUtil.getBody(l);
    AclEntry[] entries = new AclEntry[l.size()-1];
    for (int i = 0; i < entries.length; i++) {
      entries[i] = AclEntry.parseAclEntry
        (SexpUtil.getNextList(abody, "entry", "acl entry"));
    }
    SexpUtil.checkDone(abody, "acl");
    return new Acl(entries);
  }
}
