/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

/**
   A SPKI authorization field: a tag and a propagate flag.

   @see AuthCert
   
   @author Sameer Ajmani
 **/
public class Auth {
  private final Tag tag;
  private final boolean propagate;

  public Auth(Tag t, boolean p)
  {
    assert(t != null): "null tag";
    tag = t;
    propagate = p;
  }

  public Tag getTag() {
    return tag;
  }

  public boolean getPropagate() {
    return propagate;
  }

  public boolean equals(Object o) {
    if (o instanceof Auth) {
      Auth a = (Auth)o;
      return tag.equals(a.tag)
        && propagate == a.propagate;
    }
    return false;
  }

  public int hashCode() {
    return tag.hashCode()
      ^ (propagate ? 1 : 0);
  }
}

