/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import java.security.*;
import java.security.cert.*;
import java.util.*;

/**
   A SPKI certification path: essentially a wrapper around the Proof
   class.

   @see Proof
   
   @author Sameer Ajmani
 **/
public class CertPath extends java.security.cert.CertPath
{
  private Proof proof;
  public CertPath(Proof p) {
    super("SPKI");
    assert(p != null): "null proof";
    proof = p;
  }
  public Proof getProof()
  {
    return proof;
  }
  public String getType()
  {
    return "SPKI";
  }
  public Iterator getEncodings()
  {
    return Arrays.asList(new String[] { "SEXP" }).iterator();
  }
  public boolean equals(Object o)
  {
    return (o instanceof CertPath)
      && proof.equals(((CertPath)o).proof);
  }
  public int hashCode()
  {
    return proof.hashCode();
  }
  public String toString()
  {
    return proof.toString();
  }
  public byte[] getEncoded()
    throws CertificateEncodingException
  {
    return proof.toByteArray();
  }
  public byte[] getEncoded(String encoding)
    throws CertificateEncodingException
  {
    if (encoding.equals("SEXP")) {
      return getEncoded();
    }
    throw new CertificateEncodingException
      ("unsupported encoding: "+encoding);
  }
  public List getCertificates()
  {
    return Arrays.asList(proof.getCertificates());
  }
}
