/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import java.security.*;
import java.security.cert.*;

/**
   Parameters to the cert path builder and validator.  Specifies the
   statement (a SPKI/SDSI cert) that needs to be proved or validated and
   the certificate store from which to fetch certificates.

   @see CertStore
   @see CertPathBuilder
   @see CertPathValidator
   
   @author Sameer Ajmani
 **/
public class CertPathParameters
  implements java.security.cert.CertPathParameters
{
  private Cert cert; // the cert to prove
  private java.security.cert.CertStore store;
  public CertPathParameters(Cert c, java.security.cert.CertStore s)
    throws InvalidAlgorithmParameterException
  {
    if (c.getSubject() instanceof Name) {
      throw new InvalidAlgorithmParameterException
        ("cert subject must not be a name");
    }
    cert = c;
    store = s;
  }

  public Cert getCert()
  {
    return cert;
  }
  
  public java.security.cert.CertStore getStore()
  {
    return store;
  }

  public Object clone()
  {
    try {
      return new CertPathParameters(cert, store);
    } catch (InvalidAlgorithmParameterException e) {
      throw (IllegalStateException)
        new IllegalStateException().initCause(e);
    }
  }
}
