/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import java.security.*;
import java.security.cert.*;
import java.util.*;

/**
   A collection-based certificate store for SPKI/SDSI certificates.
   
   @author Sameer Ajmani
 **/
public class CertStore extends CertStoreSpi
{
  MultiMap auth = new MultiMap(); // issuer -> (issuer -> RHS) 
  MultiMap name = new MultiMap(); // name -> (name -> RHS)
  MultiMap compatible = new MultiMap(); // name -> (LHS -> name+X)
  MultiMap subject = new MultiMap(); // subject -> (LHS -> subject)
  private void init(CollectionCertStoreParameters params)
  {
    Iterator i = params.getCollection().iterator();
    while (i.hasNext()) {
      jsdsi.Certificate c = (jsdsi.Certificate)i.next();
      if (c.getCert().getSubject() instanceof Name) {
        Name n = (Name)c.getCert().getSubject();
        compatible.put(n.prefix(), c);
      } else {
        // subject is not a name
        subject.put(c.getCert().getSubject(), c);
      }
      if (c.getCert() instanceof NameCert) {
        NameCert nc = (NameCert)c.getCert();
        name.put(nc.getFullName(), c);
      }
      if (c.getCert() instanceof AuthCert) {
        auth.put(c.getCert().getIssuer(), c);
      }
    }
  }
  public CertStore(CollectionCertStoreParameters params)
    throws InvalidAlgorithmParameterException
  {
    super(params);
    init(params);
  }
  public CertStore(CertStoreParameters params)
    throws InvalidAlgorithmParameterException
  {
    super(params);
    try {
      init((CollectionCertStoreParameters)params);
    } catch (ClassCastException e) {
      throw (InvalidAlgorithmParameterException)
        new InvalidAlgorithmParameterException().initCause(e);
    }
  }

  public Collection
    engineGetCertificates(java.security.cert.CertSelector s)
    throws CertStoreException
  {
    if (!(s instanceof jsdsi.CertSelector)) {
      throw new CertStoreException("requires jsdsi.CertSelector");
    }
    return engineGetCertificates((jsdsi.CertSelector)s);
  }

  public Collection
    engineGetCertificates(jsdsi.CertSelector s)
    throws CertStoreException
  {
    if (s instanceof SubjectCertSelector) {
      return subject.get(((SubjectCertSelector)s).getSubject());
    }
    if (s instanceof CompatibleCertSelector) {
      return compatible.get(((CompatibleCertSelector)s).getFullName());
    }
    if (s instanceof NameCertSelector) {
      return name.get(((NameCertSelector)s).getFullName());
    }
    if (s instanceof AuthCertSelector) {
      return auth.get(((AuthCertSelector)s).getIssuer());
    }
    throw new CertStoreException
      ("unrecognized selector: "+s.getClass().getName());
  }
  
  public Collection
    engineGetCRLs(java.security.cert.CRLSelector s)
    throws CertStoreException
  {
    throw new UnsupportedOperationException();
  }
}
