/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

/**
   Selects all certificates whose subject is a name that starts with the
   specified issuer and string.  That is, if the issuer is K and the
   string is S, this selects all certificates of the form (LHS -> "K S
   ..."), where LHS is the left-hand-side of the certificate and "K S
   ..." is a name.
   
   @author Sameer Ajmani
 **/
public class CompatibleCertSelector extends CertSelector
{
  Principal issuer;
  String name;
  public CompatibleCertSelector(Principal i, String n)
  {
    issuer = i;
    name = n;
  }
  public Object clone()
  {
    return new CompatibleCertSelector(issuer, name);
  }
  public boolean match(jsdsi.Certificate cert)
  {
    return (cert.getCert().getSubject() instanceof Name)
      && ((Name)cert.getCert().getSubject())
      .getIssuer().samePrincipalAs(issuer)
      && ((Name)cert.getCert().getSubject())
      .getNames()[0].equals(name);
  }
  public Principal getIssuer()
  {
    return issuer;
  }
  public String getName()
  {
    return name;
  }
  public Name getFullName()
  {
    return new Name(issuer, name);
  }
}
