/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;
import jsdsi.sexp.*;
import java.util.*;

/**
   A hash operation that instructs a verifier to hash an object for
   later reference.
   
   @author Sameer Ajmani
 **/
public class HashOp extends Op
{
  private final String algo;

  public HashOp(String a)
  {
    assert(a != null): "null algo";
    algo = a;
  }

  public String getAlgorithm()
  {
    return algo;
  }

  public boolean equals(Object o)
  {
    return (o instanceof HashOp)
      && algo.equals(((HashOp)o).algo);
  }

  public int hashCode()
  {
    return algo.hashCode();
  }

  public SexpList toSexp()
  {
    Sexp[] ss = new Sexp[2];
    ss[0] = SexpUtil.toSexp("hash");
    ss[1] = SexpUtil.toSexp(getAlgorithm());
    return SexpUtil.toSexp("do", ss);
  }
  
  static HashOp parseHashOp(Iterator obody)
    throws SexpParseException
  {
    String algo = SexpUtil.getNextString(obody, "hash algo");
    SexpUtil.checkDone(obody, "op hash");
    return new HashOp(algo);
  }
}
