/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

/**
   Selects all name certificates issued by the given principal to define
   the given string.
   
   @author Sameer Ajmani
 **/
public class NameCertSelector extends CertSelector
{
  Principal issuer;
  String name;
  public NameCertSelector(Principal i, String n)
  {
    issuer = i;
    name = n;
  }
  public Object clone()
  {
    return new NameCertSelector(issuer, name);
  }
  public boolean match(jsdsi.Certificate cert)
  {
    return (cert.getCert() instanceof NameCert)
      && cert.getCert().getIssuer().samePrincipalAs(issuer)
      && ((NameCert)cert.getCert()).getName().equals(name);
  }
  public Principal getIssuer()
  {
    return issuer;
  }
  public String getName()
  {
    return name;
  }
  public Name getFullName()
  {
    return new Name(issuer, name);
  }
}
