/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.io.*;

/**
   Common superclass of all SPKI/SDSI objects.
   
   @author Sameer Ajmani
 **/
public abstract class Obj
{
  public abstract boolean equals(Object o);
  public abstract int hashCode();
  public abstract SexpList toSexp();
  
  public String toString()
  {
    try {
      StringWriter sw = new StringWriter();
      this.toSexp().writeReadable(sw, 0, 72, 0);
      return sw.toString();
    } catch (IOException e) {
      throw new Error(e);
    }
  }
  
  public byte[] toByteArray()
  {
    try {
      ByteArrayOutputStream bos = new ByteArrayOutputStream();
      this.toSexp().writeCanonical(bos);
      return bos.toByteArray();
    } catch (IOException e) {
      throw new Error(e);
    }
  }

  public static Obj parseObj(Sexp s)
    throws SexpParseException
  {
    return parseObj(SexpUtil.getList(s));
  }
  
  public static Obj parseObj(SexpList l)
    throws SexpParseException
  {
    String type = l.getType();
	if (type.equals("acl"))
	    return Acl.parseAcl(l);
	if (type.equals("entry"))
	    return AclEntry.parseAclEntry(l);
	if (type.equals("cert"))
	    return Cert.parseCert(l);
	if (type.equals("sequence"))
	    return Sequence.parseSequence(l);
	if (type.equals("public-key"))
	    return PublicKey.parsePublicKey(l);
	if (type.equals("do"))
	    return Op.parseOp(l);
	if (type.equals("signature"))
	    return Signature.parseSignature(l);
	if (type.equals("hash"))
	    return Hash.parseHash(l);
	if (type.equals("object-hash"))
	    return ObjectHash.parseObjectHash(l);
	if (type.equals("tag"))
	    return Tag.parseTag(l);
	if (type.equals("k-of-n"))
	    return Threshold.parseThreshold(l);
	if (type.equals("name"))
	    return Name.parseName(l);
	if (type.equals("principal"))
	    return Principal.parsePrincipal(l);
	if (type.equals("online"))
	    return OnlineTest.parseOnlineTest(l);
	if (type.equals("valid"))
	    return Validity.parseValidity(l);
    
    throw new SexpParseException
      ("unrecognized object type: "+type);
  }
}
