/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   The cryptographic hash of an object.
   
   @author Sameer Ajmani
 **/
public class ObjectHash extends Obj implements Subject
{
  private final Hash hash;

  public ObjectHash(Hash h)
  {
    assert(h != null): "null hash";
    hash = h;
  }

  public Hash getHash()
  {
    return hash;
  }

  public boolean isHashOf(Obj o)
  {
    Hash h = new Hash(hash.getAlgorithm(), o, hash.getURLs());
    return hash.equals(h);
  }

  public boolean equals(Object o)
  {
    return (o instanceof ObjectHash)
      && hash.equals(((ObjectHash)o).hash);
  }
  
  public int hashCode()
  {
    return hash.hashCode();
  }

  public SexpList toSexp()
  {
    Sexp[] ss = new Sexp[1];
    ss[0] = getHash().toSexp();
    return SexpUtil.toSexp("object-hash", ss);
  }
  
  static ObjectHash parseObjectHash(SexpList l)
    throws SexpParseException
  {
    Iterator obody = SexpUtil.getBody(l);
    Hash hash = Hash.parseHash(SexpUtil.getNextList(obody, "object hash"));
    SexpUtil.checkDone(obody, "object-hash");
    return new ObjectHash(hash);
  }
}
