/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;
import jsdsi.sexp.*;
import java.util.*;
import java.net.URL;

/**
   Verifies whether a certificate is valid by contacting an online
   principal or its agent.
   
   @author Sameer Ajmani
 **/
public abstract class OnlineTest extends Obj
{
  private final Principal principal;
  private final URL[] urls; // may be null

  public OnlineTest(Principal p, URL[] u)
  {
    assert(p != null): "null principal";
    principal = p;
    urls = u;
  }

  public Principal getPrincipal()
  {
    return principal;
  }

  public URL[] getURLs()
  {
    return urls;
  }

  public boolean equals(Object o)
  {
    if (o instanceof OnlineTest) {
      OnlineTest t = (OnlineTest)o;
      return principal.equals(t.principal)
        && Util.equals(urls, t.urls);
    }
    return false;
  }

  public int hashCode()
  {
    return principal.hashCode()
      ^ Util.hashCode(urls);
  }

  abstract String getSexpType();
  abstract Sexp[] getSexpParts();
  
  public final SexpList toSexp()
  {
    List l = new ArrayList();
    l.add(SexpUtil.toSexp(getSexpType()));
    l.add(SexpUtil.toSexp(getURLs()));
    l.add(getPrincipal().toSexp());
    Sexp[] parts = getSexpParts();
    for (int i = 0; i < parts.length; i++) {
      l.add(parts[i]);
    }
    return SexpUtil.toSexp("online", l);
  }
  
  static OnlineTest parseOnlineTest(SexpList l)
    throws SexpParseException
  {
    Iterator obody = SexpUtil.getBody(l);
    String type = SexpUtil.getNextString(obody, "online test type");
    URL[] urls = SexpUtil.parseURLs
      (SexpUtil.getNextList(obody, "uri", "online test uris"));
    Principal principal = Principal.parsePrincipal
      (SexpUtil.getNextList(obody, "online test principal"));
    if (type.equals("crl"))
      return Revocation.parseRevocation(principal, urls, obody);
    if (type.equals("reval"))
      return Revalidation.parseRevalidation(principal, urls, obody);
    if (type.equals("one-time"))
      return OneTime.parseOneTime(principal, urls, obody);    
    throw new SexpParseException("unrecognized online test type: "+type);    
  }
}
