/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   A tag that matches all strings with a given prefix.
   
   @author Sameer Ajmani
 **/
public class PrefixTag extends ExprTag
{
  private final String prefix;

  public PrefixTag(String p)
  {
    assert(p != null): "null prefix";
    prefix = p;
  }
  
  boolean implies(Tag that)
  {
    if ((that instanceof StringTag)
        && ((StringTag)that).getValue().startsWith(this.prefix)) {
      return true;
    }
    if ((that instanceof PrefixTag)
        && ((PrefixTag)that).prefix.startsWith(this.prefix)) {
      return true;
    }
    return false;
  }

  public boolean equals(Object that)
  {
    return (that instanceof PrefixTag)
      && this.prefix.equals(((PrefixTag)that).prefix);
  }

  public int hashCode()
  {
    return prefix.hashCode();
  }

  public String getPrefix()
  {
    return prefix;
  }

  public Sexp toTagSexp()
  {
    Sexp[] ss = new Sexp[2];
    ss[0] = SexpUtil.toSexp("prefix");
    ss[1] = SexpUtil.toSexp(getPrefix());
    return SexpUtil.toSexp("*", ss);    
  }

  static PrefixTag parsePrefixTag(Iterator tbody)
    throws SexpParseException
  {
    String p = SexpUtil.getNextString(tbody, "prefix tag");
    SexpUtil.checkDone(tbody, "prefix tag");
    return new PrefixTag(p);
  }
}
