/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;

/**
   A principal: an entity that can define names and can grant and
   receive authorizations.  Represented by a public key or its hash.
   
   @author Sameer Ajmani
 **/
public abstract class Principal extends Obj
  implements Subject, java.security.Principal
{
  public abstract boolean samePrincipalAs(Principal p);

  // for java.security.Principal
  public String getName() {
    return toString();
  }
  
  static Principal parsePrincipal(SexpList l)
    throws SexpParseException
  {
    // a principal doesn't have it's own "principal" block
    String type = l.getType();
    if (type.equals("public-key"))
      return PublicKey.parsePublicKey(l);
    if (type.equals("hash"))
      return PublicKeyHash.parsePublicKeyHash(l);

    throw new SexpParseException("unrecognized principal type: "+type);    
  }
}

