/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;

/**
   The cryptographic hash of a public key.
   
   @author Sameer Ajmani
 **/
public class PublicKeyHash extends Principal
{
  private final Hash hash;

  public PublicKeyHash(Hash h)
  {
    assert(h != null): "null hash";
    hash = h;
  }

  public Hash getHash()
  {
    return hash;
  }

  public boolean samePrincipalAs(Principal p)
  {
    if (p == null) {
      return false;
    }
    if (p instanceof PublicKeyHash) {
      return equals(p);
    }
    // p is a public key, so check that hashes match
    Hash h = new Hash(hash.getAlgorithm(), p, hash.getURLs());
    return hash.equals(h);
  }
  
  public boolean equals(Object o)
  {
    return (o instanceof PublicKeyHash)
      && hash.equals(((PublicKeyHash)o).hash);
  }
  
  public int hashCode()
  {
    return hash.hashCode();
  }

  public SexpList toSexp()
  {
    return getHash().toSexp();
  }
  
  public static PublicKeyHash parsePublicKeyHash(SexpList l)
    throws SexpParseException
  {
    return new PublicKeyHash(Hash.parseHash(l));
  }
}
