/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   A tag that specifies a set of allowed values.
   
   @author Sameer Ajmani
 **/
public class SetTag extends ExprTag
{
  private final ExprTag[] elements; // FIXME: should be a Set

  public SetTag(ExprTag[] e)
  {
    assert(e != null): "null elements";
    elements = e;
  }

  boolean implies(Tag that)
  {
    return false; // TODO: implement
  }

  public boolean equals(Object that)
  {
    return (that instanceof SetTag)
      && Util.equals(elements, ((SetTag)that).elements);
  }

  public int hashCode()
  {
    return Util.hashCode(elements);
  }

  public ExprTag[] getElements()
  {
    return elements;
  }

  public Sexp toTagSexp()
  {
    Sexp[] ss = new Sexp[elements.length+1];
    ss[0] = SexpUtil.toSexp("set");
    for (int i = 0; i < elements.length; i++) {
      ss[i+1] = elements[i].toTagSexp();
    }
    return SexpUtil.toSexp("*", ss);
  }

  static SetTag parseSetTag(Iterator tbody)
    throws SexpParseException
  {
    ArrayList l = new ArrayList();
    while (tbody.hasNext()) {
      l.add(ExprTag.parseExprTag(SexpUtil.getNext(tbody, "set tag")));
    }
    SexpUtil.checkDone(tbody, "set tag"); // sanity check
    ExprTag[] els = new ExprTag[l.size()];
    l.toArray(els);
    return new SetTag(els);
  }
}
