/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   A tag that contains a string type and a sequence of other tags.
   
   @author Sameer Ajmani
 **/
public class SimpleTag extends ExprTag
{
  private final String value;
  private final ExprTag[] tags;

  public SimpleTag(String v, ExprTag[] t)
  {
    assert(v != null): "null value";
    assert(t != null): "null tags";    
    value = v;
    tags = t;
  }

  boolean implies(Tag that)
  {
    return false; // TODO: implement
  }

  public boolean equals(Object that)
  {
    return (that instanceof SimpleTag)
      && this.value.equals(((SimpleTag)that).value)
      && Util.equals(this.tags, ((SimpleTag)that).tags);
  }

  public int hashCode()
  {
    return Util.hashCode(value) ^ Util.hashCode(tags);
  }

  public String getValue()
  {
    return value;
  }

  public ExprTag[] getTags()
  {
    return tags;
  }

  public Sexp toTagSexp()
  {
    Sexp[] ss = new Sexp[tags.length];
    for (int i = 0; i < tags.length; i++) {
      ss[i] = tags[i].toTagSexp();
    }
    return SexpUtil.toSexp(getValue(), ss);
  }

  static SimpleTag parseSimpleTag(SexpList l)
    throws SexpParseException
  {
    String value = l.getType();
    Iterator lbody = SexpUtil.getBody(l);
    ExprTag[] tags = new ExprTag[l.size()-1];
    for (int i = 0; i < tags.length; i++) {
      tags[i] = ExprTag.parseExprTag(SexpUtil.getNext(lbody, "simple tag"));
    }
    SexpUtil.checkDone(lbody, "simple tag");
    return new SimpleTag(value, tags);
  }
}
