/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   A tag defined by a simple string.
   
   @author Sameer Ajmani
 **/
public class StringTag extends ExprTag
{
  private final String value;

  public StringTag(String v)
  {
    assert(v != null): "null value";
    value = v;
  }
  
  boolean implies(Tag that)
  {
    return this.equals(that);
  }

  public boolean equals(Object that)
  {
    return (that instanceof StringTag)
      && this.value.equals(((StringTag)that).value);
  }

  public int hashCode()
  {
    return value.hashCode();
  }

  public String getValue()
  {
    return value;
  }

  public Sexp toTagSexp()
  {
    return SexpUtil.toSexp(getValue());
  }
}

