/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

import jsdsi.sexp.*;
import java.util.*;

/**
   Specifies a particular authorization permission.
   
   @author Sameer Ajmani
 **/
public abstract class Tag extends Obj
{
  /**
     A Tag that conveys all permissions.
  **/
  public static Tag ALL_TAG = new Tag()
  {
    boolean implies(Tag that)
    {
        return true;
    }
    
    public boolean equals(Object that)
    {
      return this == that;
    }

    public int hashCode()
    {
      return 1;
    }

    public Sexp toTagSexp()
    {
      return SexpUtil.toSexpList("*");
    }
    };
  
  
  /**
     A Tag that conveys no permissions.
  **/
  public static Tag NULL_TAG = new Tag()
  {
    boolean implies(Tag that)
    {
      return false;
    }
    
    public boolean equals(Object that)
    {
      return this == that;
    }
      
    public int hashCode()
    {
      return 0;
    }
    
    public Sexp toTagSexp()
    {
      throw new Error("Cannot convert NULL_TAG to Sexp");
    }
    };
  
  Tag intersect(Tag that)
  {
    if (this.equals(ALL_TAG)) {
      return that;
    }
    if (that.equals(ALL_TAG)) {
      return this;
    }
    if (this.equals(NULL_TAG)) {
      return that;
    }
    if (that.equals(NULL_TAG)) {
      return this;
    }
    if (this.equals(that)) {
      return that;
    }
    // TODO: implement full tag intersection
    return NULL_TAG;
  }
  
  abstract boolean implies(Tag that);
  abstract Sexp toTagSexp();
  
  public final SexpList toSexp()
  {
    Sexp[] ss = new Sexp[1];
    ss[0] = toTagSexp();
    return SexpUtil.toSexp("tag", ss);
  }

  static Tag parseTag(SexpList l)
    throws SexpParseException
  {
    SexpUtil.checkType(l, "tag");    
    Iterator tbody = SexpUtil.getBody(l);
    Sexp s = SexpUtil.getNext(tbody, "tag body");
    if (s instanceof SexpList) {
      SexpList ll = (SexpList)s;
      if (ll.getType().equals("*") && (ll.size() == 1)) {
        return ALL_TAG;
      }
    }
    return ExprTag.parseExprTag(s);
  }
}
