/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi;

/**
   Static utility methods.
   
   @author Sameer Ajmani
 **/
class Util {
  /**
     Returns true iff both parameters are null or they are equals().
  **/
  static boolean equals(Object a, Object b) {    
    if (a == null && b == null) {
      return true;
    }
    if (a == null || b == null) {
      return false;
    }
    if ((a instanceof Object[]) && (b instanceof Object[])) {
      return equals((Object[])a, (Object[])b);
    }
    return a.equals(b);
  }

  /**
     Returns true iff both parameters are null or if elements are equals().
  **/
  static boolean equals(Object[] a, Object[] b) {
    if (a == null && b == null) {
      return true;
    }
    if (a == null || b == null) {
      return false;
    }
    if (a.length != b.length) {
      return false;
    }
    for (int i = 0; i < a.length; i++) {
      if (!equals(a[i], b[i])) {
        return false;
      }
    }
    return true;
  }

  static int hashCode(Object o) {
    if (o == null) {
      return 0;
    }
    if (o instanceof Object[]) {
      return hashCode((Object[])o);
    }
    return o.hashCode();
  }

  static int hashCode(Object[] oa) {
    if (oa == null) {
      return 0;
    }
    int hc = 0;
    for (int i = 0; i < oa.length; i++) {
      hc = hc ^ hashCode(oa[i]);
    }
    return hc;
  }
}
