/*
  Copyright 2002 Massachusetts Institute of Technology
    
  Permission to use, copy, modify, and distribute this program for any
  purpose and without fee is hereby granted, provided that this
  copyright and permission notice appear on all copies and supporting
  documentation, the name of M.I.T. not be used in advertising or
  publicity pertaining to distribution of the program without specific
  prior permission, and notice be given in supporting documentation that
  copying and distribution is by permission of M.I.T.  M.I.T. makes no
  representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.
*/
package jsdsi.sexp;

import java.io.*;
import java.util.*;

/**
   Writes serialized S-expressions to an underlying stream.  Supports
   the canonical, transport, and readable S-expression encodings.
   
   @author Sameer Ajmani
   @see SexpInputStream
**/
public class SexpOutputStream extends FilterOutputStream {
  /**
     Creates a new SexpOutputStream that writes to the given stream.
  **/
  public SexpOutputStream(OutputStream os)
  {
    super(os);
  }
  /**
     Writes an S-expression to the stream in readable form.
     @param s the S-expression to write
     @param offset spaces indented from left
     @param width total width of window, in characters
     @param last spaces reserved on right (e.g., for closing parens)
     @throws IOException if there is an IO error
  **/
  public void writeReadable(Sexp s, int offset, int width, int last)
    throws IOException
  {
    Writer w = new OutputStreamWriter(out);
    s.writeReadable(w, offset, width, last);
    w.flush();
  }
  /**
     Writes an S-expression to the stream in transport form.
     @param s the S-expression to write
     @throws IOException if there is an IO error
  **/
  public void writeTransport(Sexp s) throws IOException
  {
    s.writeTransport(out);
  }
  /**
     Writes an S-expression to the stream in canonical form.
     @param s the S-expression to write
     @throws IOException if there is an IO error
  **/
  public void writeCanonical(Sexp s) throws IOException
  {
    s.writeCanonical(out);
  }
}
